﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinDock;
using System.Drawing;

namespace WD_AdjustCaptionHeight
{
    class CustomCreationFilter : IUIElementCreationFilter
    {
        public delegate void CustomButtonClickHandler(object sender, UIElementEventArgs e);
        public event CustomButtonClickHandler OnCustomButtonClickHandler;

        public void AfterCreateChildElements(UIElement parent)
        {
            if (parent is PaneCaptionUIElement)
            {
                PaneCaptionTextAreaUIElement textEl = parent.GetDescendant(typeof(PaneCaptionTextAreaUIElement)) 
                                                                        as PaneCaptionTextAreaUIElement;

                List<PaneCaptionButtonUIElement> buttons = parent.ChildElements.
                    Where((x) => x is PaneCaptionButtonUIElement).Cast<PaneCaptionButtonUIElement>().ToList();
                if( parent.GetDescendant(typeof(ButtonUIElement)) == null)
                {
                    ButtonUIElement button = new ButtonUIElement(parent);

                    //rectangle if there are other pane buttons
                    if (buttons.Count > 0)
                    {
                        button.Rect = new Rectangle(buttons.Last().Rect.X - buttons.Last().Rect.Width + 1, buttons.Last().Rect.Y,
                            buttons.Last().Rect.Width, buttons.Last().Rect.Height );
                    }
                    //rectangle if there are no other pane buttons
                    else
                    {
                        button.Rect = new Rectangle(parent.Rect.X + parent.Rect.Width - parent.Rect.Height -1,
                        parent.Rect.Y +1, parent.Rect.Height, parent.Rect.Height );
                    }

                    parent.ChildElements.Add(button);
                    button.ElementClick += CustomButtonClick; 
                    if (textEl != null)
                    {
                        textEl.Rect = new Rectangle(textEl.Rect.Location,
                                    new Size(textEl.Rect.Width - button.Rect.Width, textEl.Rect.Height));
                    }
                }
            }
        }

        private void CustomButtonClick(object sender, UIElementEventArgs e)
        {
            if (OnCustomButtonClickHandler != null)
            {
                OnCustomButtonClickHandler.Invoke(sender, e);
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }
    }
}
